<#
.SYNOPSIS

This script file is designed to deploy Sitecore.Universal.Tracking collection service to Azure.
.DESCRIPTION

The purpose of this script is to deploy Sitecore.Universal.Tracking collection service from WDP to Azure.
The deployment process consist of 2 steps:
1. Deployment of the WDP into Azure storage
2. Deployment of the WebApp from WDP
.EXAMPLE

PS> .\AzureDeploy.ps1 -subscriptionId "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -wdpPackagePath "C:\deployment-package\Sitecore.Tracking.Service.x.y.z.rev-ddmmyy.azure-wdp.zip" -azureSecurityKey "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" -azureClientID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -azureTenantID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -licenseXmlPath "C:\sitecorelicense\license.xml"
.EXAMPLE

PS> .\AzureDeploy.ps1 -subscriptionId "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -resourceGropuName "myResourceGroup" -resourceGroupLocation "North Europe" -wdpPackagePath "C:\deployment-package\Sitecore.Tracking.Service.x.y.z.rev-ddmmyy.azure-wdp.zip" -storageContainerName "myStorageContainer" -azureSecurityKey "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" -azureClientID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -azureTenantID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
.PARAMETER subscriptionId

The id of Azure subscription.
.PARAMETER resourceGroupName

The name of Azure resource group.
Default value: ScUT-RG
.PARAMETER resourceGroupLocation

The location of Azure resource group.
Default value: North Europe
.PARAMETER wdpPackagePath

The path to the WDP .zip archive to deploy service from.
.PARAMETER storageContainerName

The name of Azure storage container.
Default value: ScUT-SA
.PARAMETER azureSecurityKey

The Azure security key.
.PARAMETER azureClientID

The Azure client ID.
.PARAMETER azureTenantID

The Azure tenant ID.
.PARAMETER licenseXmlPath

Full path to Sitecore license.xml file.
.PARAMETER storageTemplate

Path to ARM template for deploying storage account.
.PARAMETER storageParameters

Path to ARM parameters template for deploying storage account.
.PARAMETER serviceTemplate

Path to ARM template for deploying Sitecore.Universal.Tracking to WebApp.
.PARAMETER serviceParameters

Path to ARM parameters template for deploying Sitecore.Universal.Tracking to WebApp.

#>
param(
    [Parameter(Mandatory = $True)]
    [string]
    $subscriptionId,

    [string]
    $resourceGroupName = "scuniversaltracking-rg",

    [string]
    $resourceGroupLocation = "North Europe",

    [Parameter(Mandatory = $True)]
    [string]
    $wdpPackagePath,

    [string]
    $storageContainerName = "scuniversaltracking-sa",

    [parameter(Mandatory = $true)]
    [string]
    $azureSecurityKey,

    [parameter(Mandatory = $true)]
    [string]
    $azureClientID,

    [parameter(Mandatory = $true)]
    [string]
    $azureTenantID,

    [parameter(Mandatory = $true)]
    [string]
    $licenseXmlPath,

    [parameter(Mandatory = $true)]
    [string]
    $dbConnectionString
)
try {
    Write-Host "Deployment Started..."

    function Get-NewParamJsonFile {
        param (
            [Parameter(Mandatory = $True)]
            [string]
            $serviceParameters,
    
            [Parameter(Mandatory = $True)]
            [string]
            $LicenseXmlPath,
    
            [Parameter(Mandatory = $True)]
            [string]
            $wdpPackage,

            [Parameter(Mandatory = $True)]
            [string]
            $connectionString
        )
        
        # Set the Parameters in Arm Template Parameters Json
        $paramJson = Get-Content $serviceParameters -Raw | ConvertFrom-Json
    
        Write-Verbose "Setting ARM template parameters..."
    
        if ($paramJson."`$schema") {
            $paramJson = $paramJson.parameters
        }
    
        # Read and Set the license.xml
        $licenseXml = Get-Content $LicenseXmlPath -Raw -Encoding UTF8
        $paramJson | Add-Member -NotePropertyName "LicenseXml" -NotePropertyValue @{ "value" = "$licenseXml" } -Force
        $paramJson | Add-Member -NotePropertyName "msDeployPackageUrl" -NotePropertyValue @{ "value" = $wdpPackage } -Force
        $paramJson | Add-Member -NotePropertyName "DatabaseConnectionString" -NotePropertyValue @{ "value" = $connectionString } -Force
    
        # Save to a temporary file
        $paramJsonFile = "temp_$([System.IO.Path]::GetRandomFileName())"
        $paramJson | ConvertTo-Json -Depth 100 | Set-Content $paramJsonFile -Encoding UTF8
        return $paramJsonFile
    }


    $storageTemplate = "$PSScriptRoot\templates\storageDeploy.template.json";
    $storageParameters = "$PSScriptRoot\templates\storageDeploy.parameters.json";
    $serviceTemplate = "$PSScriptRoot\templates\serviceDeploy.template.json";
    $serviceParameters = "$PSScriptRoot\templates\serviceDeploy.parameters.json";


    $secpasswd = ConvertTo-SecureString $azureSecurityKey -AsPlainText -Force;
    $mycreds = New-Object System.Management.Automation.PSCredential ($azureClientID, $secpasswd);
    Login-AzureRmAccount -ServicePrincipal -Tenant $azureTenantID -Credential $mycreds;

    Select-AzureRmSubscription -SubscriptionId $subscriptionId

    # Check whether Azure resource group exist, if not it will be created
    $resourceGroup = Get-AzureRmResourceGroup -Name $resourceGroupName -ev notPresent -ea 0;
    if (!$resourceGroup) {
        Write-Output("Creating resource group '$resourceGroupName' in location '$resourceGroupLocation'.");
        New-AzureRmResourceGroup -Name $resourceGroupName -Location $resourceGroupLocation
    }
    else {
        Write-Output("Using existing resource group '$resourceGroupName'.");
    }

    # Deploy Azure storage account from ARM template
    New-AzureRmResourceGroupDeployment -ResourceGroupName $resourceGroupName `
        -TemplateFile "$storageTemplate" `
        -TemplateParameterFile "$storageParameters";

    $storageAccountName = (Get-AzureRmResourceGroupDeployment -ResourceGroupName $resourceGroupName -Name "storageDeploy.template").Outputs.storageAccountName.value;
    $storageAccount = Get-AzureRmStorageAccount -ResourceGroupName $resourceGroupName -AccountName $storageAccountName;
    $storageAccountContext = $storageAccount.Context;

    # Check whether Azure storage contaner exist, if not it will be created
    $storageContainer = Get-AzureStorageContainer -Name $storageContainerName -Context $storageAccountContext -ev notPresent -ea 0;
    if (!$storageContainer) {
        Write-Output("Creating storage container '$storageContainerName' in storage account '$storageAccountName'.");
        New-AzureStorageContainer -Name $storageContainerName -Context $storageAccountContext -Permission blob;
    }
    else {
        Write-Output("Using existing storage container '$storageContainerName' in storage account '$storageAccountName'.");
    }

    # Deploy WDP package to storage container. If the file with same name already exist in container, it will be overwritten    
    $wdpPackageName = Split-Path $wdpPackagePath.ToLower() -Leaf

    Set-AzureStorageBlobContent -File $wdpPackagePath.ToLower() -Container $storageContainerName -Blob $wdpPackageName -Context $storageAccountContext -Force

    $azureBlobSASToken = New-AzureStorageBlobSASToken -Context $storageAccountContext -Container $storageContainerName -Blob $wdpPackageName -Permission rwd;
    $wdpPackageUrl = "https://$storageAccountName.blob.core.windows.net/$storageContainerName/$wdpPackageName$azureBlobSASToken";

    
    $paramJsonFile = Get-NewParamJsonFile -serviceParameters $serviceParameters -LicenseXmlPath $licenseXmlPath -wdpPackage $wdpPackageUrl -connectionString $dbConnectionString
    Write-Verbose "ARM template parameters are set!"

    Write-Verbose "Start deployment to webapps ..."
    New-AzureRmResourceGroupDeployment -Name  ((Get-ChildItem $serviceTemplate).BaseName + '-' + ((Get-Date).ToUniversalTime()).ToString('MMdd-HHmm'))`
            -ResourceGroupName $resourceGroupName `
			-TemplateFile $serviceTemplate `
            -TemplateParameterFile $paramJsonFile `
            -Verbose
}
catch {
    Write-Error $_.Exception.Message
         Break
}
finally {
    if ($paramJsonFile) {
        Remove-Item $paramJsonFile
    }
}
