param
(
    [Parameter(Mandatory = $True)]
    [string]
    $subscriptionId,
    
    [string]
    $resourceGroupName = "scuniversaltracking-rg",
    
    [string]
    $resourceGroupLocation = "North Europe",
    
    [string]
    $storageContainerName = "scuniversaltracking-sa",
    
    [parameter(Mandatory = $true)]
    [string]
    $azureSecurityKey,
    
    [parameter(Mandatory = $true)]
    [string]
    $azureClientID,
    
    [parameter(Mandatory = $true)]
    [string]
    $azureTenantID,

    [parameter(Mandatory = $true)]
    [string]
    $sourceDacFile,

    [parameter(Mandatory = $true)]
    [System.String]
    $targetServerName,

    [parameter(Mandatory = $true)]
    [System.String]
    $targetDatabaseName,

    [parameter(Mandatory = $true)]
    [System.String]
    $targetUser,

    [parameter(Mandatory = $true)]
    [System.String]
    $targetUserPassword
)

function Get-NewParamJsonFile {
    param (
        [Parameter(Mandatory = $True)]
        [string]
        $databaseParameters,

        [Parameter(Mandatory = $True)]
        [string]
        $adminLoginPassword,

        [Parameter(Mandatory = $True)]
        [string]
        $adminLogin,

        [Parameter(Mandatory = $True)]
        [string]
        $sqlserverName,

        [Parameter(Mandatory = $True)]
        [string]
        $dbName
    )
    
    # Set the Parameters in Arm Template Parameters Json
    $paramJson = Get-Content $databaseParameters -Raw | ConvertFrom-Json

    Write-Verbose "Setting ARM template parameters..."

    if ($paramJson."`$schema") {
        $paramJson = $paramJson.parameters
    }

    $paramJson | Add-Member -NotePropertyName "adminPassword" -NotePropertyValue @{ "value" = "$adminLoginPassword" } -Force
    $paramJson | Add-Member -NotePropertyName "adminLogin" -NotePropertyValue @{ "value" = $adminLogin } -Force
    $paramJson | Add-Member -NotePropertyName "sqlServerName" -NotePropertyValue @{ "value" = $sqlserverName } -Force
    $paramJson | Add-Member -NotePropertyName "dbName" -NotePropertyValue @{ "value" = $dbName } -Force

    # Save to a temporary file
    $paramJsonFile = "temp_$([System.IO.Path]::GetRandomFileName())"
    $paramJson | ConvertTo-Json -Depth 100 | Set-Content $paramJsonFile -Encoding UTF8
    return $paramJsonFile
}

function Set-AzureFirewallRule {
    param
    (
        [String] [Parameter(Mandatory = $true)] $ServerName,
        [String] [Parameter(Mandatory = $true)] $resourceGroupName,
        [String] $AzureFirewallName = "AzureWebAppFirewall"
    )

    $ErrorActionPreference = 'Stop'
    Write-Host "START SETTING UP FIREWALL RULE ..."
    function New-AzureSQLServerFirewallRule {
        $agentIP = (New-Object net.webclient).downloadstring("http://checkip.dyndns.com") -replace "[^\d\.]"
        New-AzureRmSqlServerFirewallRule -FirewallRuleName $AzureFirewallName -StartIpAddress $agentIP -EndIpAddress $agentIP -ResourceGroupName $resourceGroupName -ServerName $ServerName
    }
    function Update-AzureSQLServerFirewallRule {
        $agentIP = (New-Object net.webclient).downloadstring("http://checkip.dyndns.com") -replace "[^\d\.]";
        Set-AzureRmSqlServerFirewallRule -FirewallRuleName $AzureFirewallName -StartIpAddress $agentIP -EndIpAddress $agentIP -ResourceGroupName $resourceGroupName -ServerName $ServerName
    }

    If ((Get-AzureRmSqlServerFirewallRule -ServerName $ServerName -FirewallRuleName $AzureFirewallName -ResourceGroupName $resourceGroupName -ErrorAction SilentlyContinue) -eq $null) {
        New-AzureSQLServerFirewallRule
    }
    else {
        Update-AzureSQLServerFirewallRule
    }
}

try {
    Import-Module "$PSScriptRoot\..\PSModules\Common.psm1"
    
    $sqlexe = Get-SqlExeInstallPath
    
    $databaseTemplate = "$PSScriptRoot\templates\sqlDatabase.template.json";
    $databaseParameters = "$PSScriptRoot\templates\sqlDatabase.parameters.json";


    $secpasswd = ConvertTo-SecureString $azureSecurityKey -AsPlainText -Force;
    $mycreds = New-Object System.Management.Automation.PSCredential ($azureClientID, $secpasswd);
    $account = Login-AzureRmAccount -ServicePrincipal -Tenant $azureTenantID -Credential $mycreds;
    Select-AzureRmSubscription -SubscriptionId $subscriptionId

    # Check whether Azure resource group exist, if not it will be created
    $resourceGroup = Get-AzureRmResourceGroup -Name $resourceGroupName -ev notPresent -ea 0;
    if (!$resourceGroup) {
        Write-Output("Creating resource group '$resourceGroupName' in location '$resourceGroupLocation'.");
        New-AzureRmResourceGroup -Name $resourceGroupName -Location $resourceGroupLocation
    }
    else {
        Write-Output("Using existing resource group '$resourceGroupName'.");
    }
    
    $paramJsonFile = Get-NewParamJsonFile -databaseParameters $databaseParameters -adminLoginPassword $targetUserPassword -adminLogin $targetUser -sqlserverName $targetServerName  -dbName  $targetDatabaseName

    Write-Verbose "Start deployment to sql database ..."
    
    New-AzureRmResourceGroupDeployment -Name  ((Get-ChildItem $databaseTemplate).BaseName + '-' + ((Get-Date).ToUniversalTime()).ToString('MMdd-HHmm'))`
        -ResourceGroupName $resourceGroupName `
        -TemplateFile $databaseTemplate `
        -TemplateParameterFile $paramJsonFile `
        -Verbose

    
    Set-AzureFirewallRule -ServerName $targetServerName -resourceGroupName $resourceGroupName
 
    & $sqlexe `
        /SourceFile:"$sourceDacFile" `
        /Action:Publish `
        /TargetServerName:"$targetServerName.database.windows.net" `
        /TargetDatabaseName:$targetDatabaseName `
        /TargetUser:$targetUser `
        /TargetPassword:$targetUserPassword
    
}
catch {
    Write-Error $_.Exception.Message
    Break
}
finally {
    if ($paramJsonFile) {
        Remove-Item $paramJsonFile
    }
}