param(
    [Parameter(Mandatory = $True)]
    [string]
    $wdpPackagePath,

    [Parameter(Mandatory = $True)]
    [string]
    $licenseXmlPath,

    [string]
    $databaseConnectionString = "NotSet",

    [string]
    $siteFolderPath = "C:\inetpub\wwwroot",

    [string]
    $instanceName = "Sitecore.Tracking.Processing.Service",

    [string]
    $IISAppPollName = "Sitecore.Tracking.Processing.Service",

    [string]
    $siteHttpPort = "80",

    [string]
    $siteHttpsPort = "443"
)

<#
    .SYNOPSIS
        Deploy IIS website from wdp package

    .PARAMETER wdpPackagePath
        Absolute path to wdp package file

    .PARAMETER licenseXmlPath
        Absolute path to license.xml file

    .PARAMETER databaseConnectionString
        Connection string to SQL that will be set after Sitecore.Universal.Tracking collection service deployment

    .PARAMETER siteFolderPath
        Absolute path to root folder where site will be created

    .PARAMETER instanceName
        IIS website name

    .PARAMETER IISAppPollName
        IIS application pool for website

    .PARAMETER siteHttpPort
        IIS website binding for http port

    .PARAMETER siteHttpsPort
        IIS website binding for https port
#>
function Deploy-WebApp {
    param (
        [Parameter(Mandatory = $True)]
        [string]
        $wdpPackagePath,

        [Parameter(Mandatory = $True)]
        [string]
        $licenseXmlPath,

        [string]
        $databaseConnectionString = "NotSet",

        [string]
        $siteFolderPath = "C:\inetpub\wwwroot",

        [string]
        $instanceName = "Sitecore.Tracking.Processing.Service",

        [string]
        $IISAppPollName = "Sitecore.Tracking.Processing.Service",

        [string]
        $siteHttpPort = "80",

        [string]
        $siteHttpsPort = "443"
    )

    try {
        Import-Module WebAdministration

        $appPool = New-Item IIS:\AppPools\$IISAppPollName -Force -ErrorAction SilentlyContinue
        $appPool | Set-ItemProperty -Name "managedRuntimeVersion" -Value ""
        Set-ItemProperty -Path IIS:\AppPools\$IISAppPollName -name "startMode" -value "AlwaysRunning"
        Set-ItemProperty -Path IIS:\AppPools\$IISAppPollName -name processmodel.idleTimeoutAction -value Suspend
        Set-ItemProperty -Path IIS:\AppPools\$IISAppPollName -name processmodel.idleTimeout -value ( [TimeSpan]::FromMinutes(0))
        Set-ItemProperty -Path IIS:\AppPools\$IISAppPollName -name processmodel.identityType -value 4 #ApplicationPoolIdentity

        $bindings = @(
            @{protocol = "http"; bindingInformation = "*:" +$siteHttpPort +":" + $instanceName },
            @{protocol = "https"; bindingInformation = "*:" +$siteHttpsPort +":" + $instanceName }
        )

        New-Item IIS:\Sites\$instanceName -physicalPath (Join-Path $siteFolderPath $instanceName) -bindings $bindings
        Set-ItemProperty IIS:\Sites\$instanceName -name applicationPool -value $IISAppPollName

        [string]$msDeployPath = (get-childitem "HKLM:\SOFTWARE\Microsoft\IIS Extensions\MSDeploy" | Select -last 1).GetValue("InstallPath");
        [string]$msDeployExePath = Join-Path $msDeployPath "msdeploy.exe";

        if (!(Test-Path $msDeployExePath)) {
            Write-Host "cannot find msdeploy.exe on machine, please refer the link to donwload and install it, link: https://www.iis.net/downloads/microsoft/web-deploy";
        }

        $setParamFilePath = Add-SetParamFile -licenseXmlPath $licenseXmlPath -databaseConnectionString $databaseConnectionString


        & $msDeployExePath `
            -verb:sync `
            -source:package=$wdpPackagePath `
            -dest:iisApp=$instanceName `
            -setParamFile="$setParamFilePath"

        $sitePath = (Join-Path $siteFolderPath "$instanceName")
        if(!(Test-Path "$sitePath\App_Data\logs")) {
            $TargetPath = "$sitePath\App_Data\logs\"
            New-Item -Path $TargetPath -ItemType Directory -Force;
            $acl = Get-Acl $TargetPath
            $accessRule = New-Object System.Security.AccessControl.FileSystemAccessRule("IIS AppPool\$instanceName", "FullControl","ContainerInherit,ObjectInherit", "None", "Allow")
            $acl.SetAccessRule($accessRule)
            Set-Acl $TargetPath $acl
        }

        return $instanceName;

    }
    catch {
        Write-Error $_.Exception.Message
        Break
    }
    finally {
        if ($setParamFilePath) {
            Remove-Item $setParamFilePath
        }
    }
}

<#
    .SYNOPSIS
        Create temp file with content of setParameters.xml file for setting MsDeploy tool parameters

    .PARAMETER licenseXmlPath
        Absolute path to license.xml file

    .PARAMETER databaseConnectionString
        Connection string to SQL that will be set after Sitecore.Universal.Tracking collection service deployment
#>
function Add-SetParamFile {
    param(
        [Parameter(Mandatory = $true)]
        [string]
        $licenseXmlPath,

        [Parameter(Mandatory = $true)]
        [string]
        $databaseConnectionString
    )

    $setParamFile = "$PSScriptRoot\setParameters.xml";
    $setParamTempFile = "temp_$([System.IO.Path]::GetRandomFileName())";
    $licenseXml = Get-Content $LicenseXmlPath -Raw -Encoding UTF8;

    $xml = New-Object XML
    $xml.Load($setParamFile)
    $node = $xml.SelectSingleNode("//parameters/setParameter[@name='LicenseXml']");
    $node.value = "$licenseXml";
    $node = $xml.SelectSingleNode("//parameters/setParameter[@name='DatabaseConnectionString']");
    $node.value = $databaseConnectionString
    $xml.Save("$PSScriptRoot\$setParamTempFile");

    return $setParamTempFile;
}

<#
    .SYNOPSIS
        Check passed parameters for existing in system, if any of them already used, throws exception

    .PARAMETER instanceName
        IIS website name

    .PARAMETER IISAppPollName
        IIS application pool name for website

    .PARAMETER siteFolderPath
        Absolute path to IIS website folder
#>
function ValidateDeploymentParameters {
    param(
        [Parameter(Mandatory=$true)]
        [string]
        $instanceName,

        [Parameter(Mandatory=$true)]
        [string]
        $IISAppPollName,

        [Parameter(Mandatory=$true)]
        [string]
        $siteFolderPath
    )

    if (Get-Website -Name $instanceName) {
        throw "Website with name $instanceName already exist. Please change instanceName parameter in script."
    }

    $iisAppPool = Get-IISAppPool $instanceName
    if ($null -ne $iisAppPool -and $null -ne $iisAppPool.State) {
        throw "IISAppPoll with name $instanceName already exist. Please change instanceName parameter in script."
    }

    if((Test-Path (Join-Path $siteFolderPath $instanceName))){
        throw "Physical path " + (Join-Path $siteFolderPath $instanceName) + " for IIS website already exist. Please change siteFolderPath or instanceName parameter in script."
    }
}


try {

    ValidateDeploymentParameters -instanceName $instanceName -IISAppPollName $IISAppPollName -siteFolderPath $siteFolderPath

    Deploy-WebApp `
        -wdpPackagePath $wdpPackagePath `
        -licenseXmlPath $licenseXmlPath `
        -databaseConnectionString $databaseConnectionString `
        -siteFolderPath $siteFolderPath `
        -instanceName $instanceName `
        -IISAppPollName $IISAppPollName `
        -siteHttpPort $siteHttpPort `
        -siteHttpsPort $siteHttpsPort `
        -Verbose
}
catch {
    Write-Error $_.Exception.Message
    Break
}