<#
    .SYNOPSIS
    Deploy Sitecore.Universal.Tracking database to SQL Server

    .DESCRIPTION
    Deploy Sitecore.Universal.Tracking DACPAC to specified SQL Server instance

    .PARAMETER serverName
    Specifies SQL Server name

    .PARAMETER dbName
    Specifies SQL Server database name

    .PARAMETER dbUser
    Specifies SQL Server user name

    .PARAMETER dbPassword
    Specifies SQL Server user password
#>
param
(
    [parameter(Mandatory = $true)]
    [string]
    $sourceDacFile,

    [parameter(Mandatory = $true)]
    [System.String]
    $serverName,

    [parameter(Mandatory = $true)]
    [System.String]
    $dbName,

    [parameter(Mandatory = $true)]
    [System.String]
    $dbUser,

    [parameter(Mandatory = $true)]
    [string]
    $dbPassword
)

Function Deploy-DatabaseDacpac {
    <#
        .SYNOPSIS
        Deploy database DACPAC to SQL Server

        .DESCRIPTION
        Deploy database DACPAC to specified SQL Server instance

        .PARAMETER dacpacPath
        Specifies path to database DACPAC file

        .PARAMETER serverName
        Specifies SQL Server name

        .PARAMETER dbName
        Specifies SQL Server database name

        .PARAMETER dbUser
        Specifies SQL Server user name

        .PARAMETER dbPassword
        Specifies SQL Server user password

        .EXAMPLE
        Deploy-DatabaseDacpac "C:\inetpub\wwwroot\TestDeploy\Databases\dacpac\Sitecore.Tracking.dacpac" ".\SQL2017" "Sitecore.Tracking"
    #>
    param(
        [parameter(Mandatory = $true)]
        [string]$dacpacPath,
        [parameter(Mandatory = $true)]
        [string]$serverName,
        [parameter(Mandatory = $true)]
        [string]$dbName,
        [parameter(Mandatory = $true)]
        [string]$dbUser,
        [parameter(Mandatory = $true)]
        [string]$dbPassword
    )

    $sqlExe = Get-SqlExeInstallPath

    & $sqlExe `
     /SourceFile:"$dacpacPath" `
     /Action:Publish `
     /TargetServerName:$serverName `
     /TargetDatabaseName:$dbName `
     /TargetUser:$dbUser `
     /TargetPassword:$dbPassword `
     /p:IncludeCompositeObjects=True `
     /p:CreateNewDatabase=False

    Write-Output "Database $dbName has been deployed to the SQL Server $serverName" -Level "Info"
}

Function Check-DatabaseExists {
    <#
        .SYNOPSIS
        Checks whether database is present on SQL Server

        .DESCRIPTION
        Checks whether database is present on specified SQL Server instance

        .PARAMETER serverName
        Specifies SQL Server name

        .PARAMETER dbName
        Specifies SQL Server database name

        .PARAMETER dbUser
        Specifies SQL Server user name

        .PARAMETER dbPassword
        Specifies SQL Server user password
    #>

    param(
        [parameter(Mandatory = $true)]
        [string]$serverName,
        [parameter(Mandatory = $true)]
        [string]$dbName,
        [parameter(Mandatory = $true)]
        [string]$dbUser,
        [parameter(Mandatory = $true)]
        [String]$dbPassword
    )

    [System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.ConnectionInfo")
    [System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.Smo")

    $ServerConnection = New-Object Microsoft.SqlServer.Management.Common.ServerConnection
    $ServerConnection.ServerInstance = $serverName
    $ServerConnection.LoginSecure = $false
    $ServerConnection.Login = $dbUser
    $ServerConnection.Password = $dbPassword

    $Server = New-Object Microsoft.SqlServer.Management.Smo.Server -ArgumentList $ServerConnection

    if ($Server.Databases.Contains($dbName)) {
        Write-Output "Database $dbName was found on the SQL Server $serverName" -Program "MS SQL"
        return $true
    }
    else {
        Write-Output "Database $dbName was not found on the SQL Server $serverName" -Program "MS SQL"
        return $false
    }

    if ($null -ne $Server) {
        Write-Output "Disconnecting Sql Server $serverName" -Program "MS SQL" -Level "Info"
        $Server.ConnectionContext.Disconnect()
    }
}

try {
    Import-Module "$PSScriptRoot\..\PSModules\Common.psm1"

    if ((Check-DatabaseExists $serverName $dbName $dbUser $dbPassword) -eq $true) {
        throw "Deployment failed. Database $dbName already exist on the SQL Server $serverName"
    }
    else {
        Deploy-DatabaseDacpac $sourceDacFile $serverName $dbName $dbUser $dbPassword
    }
}
catch {
    $ErrorMessage = $_.Exception.Message
    write-output $ErrorMessage
}
