param(
    [Parameter(Mandatory = $True)]
    [string]
    $wdpPackagePath,

    [Parameter(Mandatory = $True)]
    $instanceFolderPath,

    [Parameter(Mandatory = $True)]
    [string]
    $licenseXmlPath,

    [string]
    $databaseConnectionString = "NotSet",

    [string]
    $serviceName = "Sitecore Universal Tracker Processing Service",

    [string]
    $serviceDisplayName = "Sitecore Universal Tracker Processing Service",

    [string]
    $serviceDescription = "Sitecore Host Based Tracking Processing Service"
)

<#
    .SYNOPSIS
        Create temp file with content of setParameters.xml file for setting MsDeploy tool parameters

    .PARAMETER licenseXmlPath
        Absolute path to license.xml file

    .PARAMETER databaseConnectionString
        Connection string to SQL that will be set after Sitecore.Universal.Tracking collection service deployment
#>
function Add-SetParamFile {
    param(
        [Parameter(Mandatory = $true)]
        [string]
        $licenseXmlPath,

        [Parameter(Mandatory = $true)]
        [string]
        $databaseConnectionString
    )

    $setParamFile = "$PSScriptRoot\setParameters.xml";
    $setParamTempFile = "temp_$([System.IO.Path]::GetRandomFileName())";
    $licenseXml = Get-Content $LicenseXmlPath -Raw -Encoding UTF8;

    $xml = New-Object XML
    $xml.Load($setParamFile)
    $node = $xml.SelectSingleNode("//parameters/setParameter[@name='LicenseXml']");
    $node.value = "$licenseXml";
    $node = $xml.SelectSingleNode("//parameters/setParameter[@name='DatabaseConnectionString']");
    $node.value = $databaseConnectionString
    $xml.Save("$PSScriptRoot\$setParamTempFile");

    return $setParamTempFile;
}

<#
    .SYNOPSIS
        Create new windows service with LocalService account

    .PARAMETER serviceName
        Windows service name

    .PARAMETER serviceDisplayName
        Windows display service name

    .PARAMETER serviceDescription
        Windows service description

    .PARAMETER pathToServiceExe
        Path to executable of windows service
#>
function Create-WindowsService {
    param (
        [Parameter(Mandatory = $true)]
        [string]
        $serviceName,

        [Parameter(Mandatory = $true)]
        [string]
        $serviceDisplayName,

        [Parameter(Mandatory = $true)]
        [string]
        $serviceDescription,

        [Parameter(Mandatory = $true)]
        [string]
        $pathToServiceExe
    )

    # Installing new windows service with LocalSystem account
    New-Service `
        -Name $serviceName `
        -DisplayName $serviceDisplayName `
        -Description $serviceDescription `
        -StartupType Automatic `
        -BinaryPathName $pathToServiceExe `
        -Verbose

    # Here trying to find installed windows service and change system account to LocalService
    $filter = "Name='$serviceName'"

    $tries = 0

    while (($service -eq $null -and $tries -le 3)) {
        if ($tries -ne 0) {
            sleep 2
        }
        $service = Get-WMIObject -namespace "root\cimv2" -class Win32_Service -Filter $filter
        $tries = $tries + 1
    }

    if ($service -eq $null) {
        throw "Could not find '$serviceName' service";
    }
    $service.Change($null, $null, $null, $null, $null, $null, "NT AUTHORITY\LocalService", "");
    $service.StopService();
    $service.StartService();
}

Import-Module WebAdministration
function Set-LogPath($baseServicePath) {
    # Patch path to the log file in case deployment as a Windows service
    # Set absolute path for the log.
    $configPath = (Resolve-Path "$baseServicePath\sitecorehost.xml").ProviderPath
    [xml]$xml = Get-Content $configPath;

    $logPathFormat = $xml.Sitecore.Logging.Serilog.WriteTo.DefaultLogger.Args.pathFormat;

    if ($logPathFormat -And !(Split-Path -Path $logPathFormat -IsAbsolute)) {
        $xml.Sitecore.Logging.Serilog.WriteTo.DefaultLogger.Args.pathFormat = (Join-Path -Path "$baseServicePath" -ChildPath "$logPathFormat").ToString();
    }

    $xml.Save($configPath);

    if(!(Test-Path "$baseServicePath\App_Data\logs")) {
        $TargetPath = "$baseServicePath\App_Data\logs\"
        New-Item -Path $TargetPath -ItemType Directory -Force;
        $acl = Get-Acl $TargetPath
        $accessRule = New-Object System.Security.AccessControl.FileSystemAccessRule("NT AUTHORITY\LocalService", "FullControl","ContainerInherit,ObjectInherit", "None", "Allow")
        $acl.SetAccessRule($accessRule)
        Set-Acl $TargetPath $acl
    }
}

try {

    if (($_ = Get-Service -Name $serviceName -ErrorAction SilentlyContinue)) {
        throw "Windows service with name $serviceName already exist. Please change serviceName parameter in script."
    }

    if((Test-Path ("$instanceFolderPath\*"))){
        throw "Physical folder " + ($instanceFolderPath) + " already exist and not empty. Please change instanceFolderPath."
    }

    [string]$msDeployPath = (get-childitem "HKLM:\SOFTWARE\Microsoft\IIS Extensions\MSDeploy" | Select -last 1).GetValue("InstallPath");
    [string]$msDeployExePath = Join-Path $msDeployPath "msdeploy.exe";

    if (!(Test-Path $msDeployExePath)) {
        Write-Host "cannot find msdeploy.exe on machine, please refer the link to donwload and install it, link: https://www.iis.net/downloads/microsoft/web-deploy";
    }

    $setParamFilePath = Add-SetParamFile -licenseXmlPath $licenseXmlPath -databaseConnectionString $databaseConnectionString

    New-Item "$instanceFolderPath" -type Directory -Force

    & $msDeployExePath `
        -verb:sync `
        -source:package=$wdpPackagePath `
        -dest:dirPath=$instanceFolderPath `
        -setParamFile="$setParamFilePath"

    if ($LASTEXITCODE) { throw "Deploy Processing is failed" }

    Set-LogPath -baseServicePath $instanceFolderPath

    Create-WindowsService `
        -serviceName $serviceName `
        -serviceDisplayName $serviceDisplayName `
        -serviceDescription $serviceDescription `
        -pathToServiceExe "$instanceFolderPath\Sitecore.Tracking.Processing.ServiceHost.exe -s"
}
catch {
    Write-Error $_.Exception.Message
    Break
}