function Get-SubKeysInFloatFormat($keys) {
    $targetKeys = @()
    foreach ($key in $keys) {
        try {
            $targetKeys += [decimal] $key
        }
        catch {}
    }

    $targetKeys
}

function Get-RegistryValueIgnoreError {
    param
    (
        [parameter(Mandatory = $true)]
        [Microsoft.Win32.RegistryHive]
        $RegistryHive,

        [parameter(Mandatory = $true)]
        [System.String]
        $Key,

        [parameter(Mandatory = $true)]
        [System.String]
        $Value,

        [parameter(Mandatory = $true)]
        [Microsoft.Win32.RegistryView]
        $RegistryView
    )

    try {
        $baseKey = [Microsoft.Win32.RegistryKey]::OpenBaseKey($RegistryHive, $RegistryView)
        $subKey = $baseKey.OpenSubKey($Key)
        if ($subKey -ne $null) {
            return $subKey.GetValue($Value)
        }
    }
    catch {
    }
    return $null
}

function Get-SqlPackageForSqlVersion([int] $majorVersion, [bool] $wow6432Node) {
    $sqlInstallRootRegKey = "SOFTWARE", "Microsoft", "Microsoft SQL Server", "$majorVersion" -join [System.IO.Path]::DirectorySeparatorChar

    if ($wow6432Node -eq $true) {
        $sqlInstallRootPath = Get-RegistryValueIgnoreError LocalMachine "$sqlInstallRootRegKey" "VerSpecificRootDir" Registry64
    }
    else {
        $sqlInstallRootPath = Get-RegistryValueIgnoreError LocalMachine "$sqlInstallRootRegKey" "VerSpecificRootDir" Registry32
    }

    if ($null -eq $sqlInstallRootPath) {
        return $null
    }

    Write-Verbose "Sql Version Specific Root Dir for version $majorVersion as read from registry: $sqlInstallRootPath"

    $DacInstallPath = [System.IO.Path]::Combine($sqlInstallRootPath, "Dac", "bin", "SqlPackage.exe")

    if (Test-Path $DacInstallPath) {
        Write-Verbose "Dac Framework installed with SQL Version $majorVersion found at $DacInstallPath on machine $env:COMPUTERNAME"
        return $DacInstallPath
    }
    else {
        return $null
    }
}

function Get-SqlExeInstallPath {

    $sqlRegKey = "HKLM:", "SOFTWARE", "Wow6432Node", "Microsoft", "Microsoft SQL Server" -join [System.IO.Path]::DirectorySeparatorChar
    $sqlRegKey64 = "HKLM:", "SOFTWARE", "Microsoft", "Microsoft SQL Server" -join [System.IO.Path]::DirectorySeparatorChar

    if (-not (Test-Path $sqlRegKey)) {
        $sqlRegKey = $sqlRegKey64
    }

    if (-not (Test-Path $sqlRegKey)) {
        return $null, 0
    }

    $keys = Get-Item $sqlRegKey | ForEach-Object { $_.GetSubKeyNames() } 
    $versions = Get-SubKeysInFloatFormat $keys | Sort-Object -Descending

    Write-Verbose "Sql Versions installed on machine $env:COMPUTERNAME as read from registry: $versions"

    foreach ($majorVersion in $versions) {
        $DacInstallPathWow6432Node = Get-SqlPackageForSqlVersion $majorVersion $true
        $DacInstallPath = Get-SqlPackageForSqlVersion $majorVersion $false

        if ($null -ne $DacInstallPathWow6432Node) {
            return $DacInstallPathWow6432Node
        }
        elseif ($null -ne $DacInstallPath) {
            return $DacInstallPath
        }
    }

    Write-Verbose "Dac Framework (installed with SQL) not found on machine $env:COMPUTERNAME"      
    return $null, 0
}